@@ -1,7 +1,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -90,8 +90,8 @@ def norm(vec):
 def norm_of(vec):
     '''
     Returns the norm of the vector.
-    Reasonably faster than Numpy version.
-    Only for 3D vectors.
+    Faster than Numpy version, but 
+    only compatible with 3D vectors.
     '''
     return sqrt((vec[0]*vec[0] + vec[1]*vec[1] + vec[2]*vec[2]))
 
