@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ def xtb_opt(
         constrained_dihedrals=None,
         constrained_dih_angles=None,
         method='GFN2-xTB',
-        maxiter=None,
+        maxiter=500,
         solvent=None,
         charge=0,
         title='temp',
@@ -171,7 +171,7 @@ def xtb_opt(
     maxiter = maxiter if maxiter is not None else 0
     s = f'$opt\n   logfile={trajname}\n   output={outname}\n   maxcycle={maxiter}\n'
          
-    if constrained_indices is not None:  
+    if constrained_indices is not None:
         # s += '\n$fix\n   atoms: '
         # for i in np.unique(np.array(constrained_indices).flatten()):
         #     s += f"{i+1},"
@@ -438,13 +438,19 @@ def energy_grepper(filename, signal_string, position):
                 raise Exception()
 
 def xtb_get_free_energy(coords, atomnos, method='GFN2-xTB', solvent=None,
-                        charge=0, title='temp', **kwargs):
+                        charge=0, title='temp', sph=False, **kwargs):
     '''
+    Calculates free energy with XTB,
+    without optimizing the provided structure.
     '''
+
     with open(f'{title}.xyz', 'w') as f:
         write_xyz(coords, atomnos, f, title=title)
 
-    s = f'$opt\n   logfile={title}_opt.log\n$end'
+    outname = 'xtbopt.xyz'
+    trajname = f'{title}_opt_log.xyz'
+    s = f'$opt\n   logfile={trajname}\n   output={outname}\n   maxcycle=1\n'
+
           
     if method.upper() in ('GFN-XTB', 'GFNXTB'):
         s += '\n$gfn\n   method=1\n'
@@ -458,7 +464,10 @@ def xtb_get_free_energy(coords, atomnos, method='GFN2-xTB', solvent=None,
     with open(f'{title}.inp', 'w') as f:
         f.write(s)
     
-    flags = '--ohess'
+    if sph:
+        flags = '--bhess'
+    else:
+        flags = '--ohess'
     
     if method in ('GFN-FF', 'GFNFF'):
         flags += ' --gfnff'
@@ -498,7 +507,7 @@ def xtb_get_free_energy(coords, atomnos, method='GFN2-xTB', solvent=None,
 
     except FileNotFoundError:
         # return 1E10
-        print(f'temp_hess.log not present here - we are in', os.getcwd())
+        # print(f'temp_hess.log not present here - we are in', os.getcwd())
         print(os.listdir())
         sys.exit()
 
@@ -597,6 +606,8 @@ def crest_mtd_search(
         method='GFN2-XTB//GFN-FF',
         solvent='CH2Cl2',
         charge=0,
+        kcal=None,
+        ncimode=False,
         title='temp',
         procs=4,
         threads=1,
@@ -633,9 +644,11 @@ def crest_mtd_search(
 
     '''
 
+    # Remove title directory, if already present
     if title in os.listdir():
         shutil.rmtree(os.path.join(os.getcwd(), title))
         
+    # make title directory and cd into it
     os.mkdir(title)
     os.chdir(os.path.join(os.getcwd(), title))
 
@@ -653,10 +666,14 @@ def crest_mtd_search(
     s = f'$opt\n   '
          
     if constrained_indices is not None:  
-        s += '\n$constrain\n   atoms: '
-        for i in np.unique(np.array(constrained_indices).flatten()):
-            s += f"{i+1},"
-        s = s[:-1] + "\n"
+        s += '\n$constrain\n'
+        # s += '   atoms: '
+        # for i in np.unique(np.array(constrained_indices).flatten()):
+        #     s += f"{i+1},"
+
+        for (c1, c2), cd in zip(constrained_indices, constrained_distances):
+            cd = "auto" if cd is None else cd
+            s += f"    distance: {c1+1}, {c2+1}, {cd}\n"
 
     if constrained_dihedrals is not None:
         assert len(constrained_dihedrals) == len(constrained_dih_angles)
@@ -698,15 +715,21 @@ def crest_mtd_search(
     with open(f'{title}.inp', 'w') as f:
         f.write(s)
     
+    # avoid restarting the run
     flags = '--norestart'
       
+    # add method flag
     if method.upper() in ('GFN-FF', 'GFNFF'):
         flags += ' --gfnff'
         # declaring the use of FF instead of semiempirical
 
+    elif method.upper() in ('GFN2-XTB', 'GFN2'):
+        flags += ' --gfn2'
+
     elif method.upper() in ('GFN2-XTB//GFN-FF', 'GFN2//GFNFF'):
         flags += ' --gfn2//gfnff'
 
+    # adding other options
     if charge != 0:
         flags += f' --chrg {charge}'
 
@@ -724,6 +747,15 @@ def crest_mtd_search(
         else:
             flags += f' --alpb {solvent}'
 
+    if kcal is None:
+        kcal = 10
+    flags += f' --ewin {kcal}'
+
+    if ncimode:
+        flags += f' --nci'
+
+    flags += ' --noreftopo'
+
     try:
         with open(f"{title}.out", "w") as f:
             check_call(f'crest {title}.xyz --cinp {title}.inp {flags}'.split(), stdout=f, stderr=STDOUT)
@@ -732,9 +764,14 @@ def crest_mtd_search(
         print('KeyboardInterrupt requested by user. Quitting.')
         sys.exit()
 
+    # if CREST crashes, cd into the parent folder before propagating the error
+    except CalledProcessError:
+        os.chdir(os.path.dirname(os.getcwd()))
+        raise CalledProcessError
+
     new_coords = read_xyz('crest_conformers.xyz').atomcoords
 
-    clean_directory((f'{title}.inp', f'{title}.xyz', f"{title}.out"))     
+    # clean_directory((f'{title}.inp', f'{title}.xyz', f"{title}.out"))     
 
     for filename in ('gfnff_topo',
                         'charges',
@@ -751,6 +788,6 @@ def crest_mtd_search(
             pass
 
     os.chdir(os.path.dirname(os.getcwd()))
-    shutil.rmtree(os.path.join(os.getcwd(), title))
+    # shutil.rmtree(os.path.join(os.getcwd(), title))
     
     return new_coords
\ No newline at end of file
