@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -52,4 +52,9 @@ class SegmentedGraphError(Exception):
 class NoOrbitalError(Exception):
     '''
     Thrown when trying to access orbital data when they are not present
+    '''
+
+class FatalError(Exception):
+    '''
+    Thrown when a molecule optimization crashed or scrambled fatally
     '''
\ No newline at end of file
