@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@ GNU General Public License for more details.
 import os
 # import pickle
 import time
-from subprocess import DEVNULL, STDOUT, check_call
+from subprocess import CalledProcessError
 
 import numpy as np
 from networkx import connected_components
@@ -28,18 +28,20 @@ from tscode.ase_manipulations import ase_saddle
 from tscode.atropisomer_module import dihedral_scan
 from tscode.automep import automep
 from tscode.calculators._xtb import crest_mtd_search
-from tscode.errors import InputError
+from tscode.errors import InputError, FatalError
 from tscode.graph_manipulations import graphize
 from tscode.hypermolecule_class import align_structures
 from tscode.mep_relaxer import ase_mep_relax
 from tscode.optimization_methods import (_refine_structures, optimize,
                                          prune_by_moment_of_inertia)
 from tscode.pka import pka_routine
-from tscode.python_functions import prune_conformers_rmsd, prune_conformers_tfd
+from tscode.python_functions import prune_conformers_tfd
+from tscode.rmsd_pruning import prune_conformers_rmsd
 from tscode.settings import (CALCULATOR, DEFAULT_FF_LEVELS, DEFAULT_LEVELS,
                              FF_CALC, FF_OPT_BOOL, PROCS)
-from tscode.torsion_module import csearch, prune_conformers_rmsd_rot_corr, _get_quadruplets
-from tscode.utils import (get_scan_peak_index, read_xyz,
+from tscode.torsion_module import (_get_quadruplets, csearch,
+                                   prune_conformers_rmsd_rot_corr)
+from tscode.utils import (get_scan_peak_index, molecule_check, read_xyz,
                           suppress_stdout_stderr, time_to_string, write_xyz)
 
 
@@ -52,15 +54,13 @@ def operate(input_string, embedder):
 
     filename = embedder._extract_filename(input_string)
    
+    if not hasattr(embedder, "t_start_run"):
+        embedder.t_start_run = time.perf_counter()
+
     if embedder.options.dryrun:
         embedder.log(f'--> Dry run requested: skipping operator \"{input_string}\"')
         return filename
 
-    if 'confab>' in input_string:
-        outname = confab_operator(filename,
-                                    embedder.options,
-                                    logfunction=embedder.log)
-
     elif 'csearch>' in input_string:
         outname = csearch_operator(filename, embedder)
 
@@ -86,10 +86,10 @@ def operate(input_string, embedder):
         scan_operator(filename, embedder)
         outname = filename
 
-    elif 'autoneb>' in input_string:
-        automep_filename = automep(embedder)
-        neb_operator(automep_filename, embedder)
-        embedder.normal_termination()
+    elif 'automep>' in input_string:
+        automep(embedder, n_images=embedder.options.images if hasattr(embedder.options, 'images') else 9)
+        # neb_operator(automep_filename, embedder)
+        # embedder.normal_termination()
 
     elif 'neb>' in input_string:
         neb_operator(filename, embedder)
@@ -108,16 +108,45 @@ def operate(input_string, embedder):
 
         data = read_xyz(filename)
 
-        ase_mep_relax(
-            embedder,
-            data.atomcoords,
-            data.atomnos,
-            title=embedder.stamp,
-            n_images=embedder.options.images if hasattr(embedder.options, 'images') else None,
-            logfunction=embedder.log,
-            write_plot=True,
-            verbose_print=True,
-            )
+        # can implement a smart safety feature that is
+        # disabled when some bonds have to be let free to break
+        no_bonds_breaking = True
+
+        if no_bonds_breaking:
+
+            mep, _, exit_status = ase_mep_relax(
+                                                embedder,
+                                                data.atomcoords,
+                                                data.atomnos,
+                                                title=embedder.stamp+"_safe",
+                                                n_images=embedder.options.images if hasattr(embedder.options, 'images') else None,
+                                                logfunction=embedder.log,
+                                                write_plot=True,
+                                                verbose_print=True,
+                                                safe=True
+                                                )
+            
+        else:
+            mep = data.atomcoords
+            exit_status = True
+        
+        if exit_status:
+
+            if no_bonds_breaking:
+                print("--> Completed safe optimization, relaxing bond distance constraints.")
+
+            ase_mep_relax(
+                embedder,
+                mep,
+                data.atomnos,
+                title=embedder.stamp,
+                n_images=embedder.options.images if hasattr(embedder.options, 'images') else None,
+                logfunction=embedder.log,
+                write_plot=True,
+                verbose_print=True,
+                safe=True
+                )
+        
         embedder.normal_termination()
 
     else:
@@ -126,47 +155,6 @@ def operate(input_string, embedder):
 
     return outname
 
-def confab_operator(filename, options, logfunction=None):
-    '''
-    '''
-
-    if logfunction is not None:
-        logfunction(f'--> Performing conformational search and optimization on {filename}')
-
-    data = read_xyz(filename)
-
-    if len(data.atomcoords) > 1:
-        raise InputError(f'Requested conformational search on file {filename} that already contains more than one structure.')
-
-    if len(tuple(connected_components(graphize(data.atomcoords[0], data.atomnos)))) > 1:
-        raise InputError((f'Requested conformational search on a molecular complex (file {filename}). '
-                           'Confab is not suited for this task, use internal csearch> operator.'))
-
-    if len(set(data.atomnos) - {1,6,7,8,9,15,16,17,35,53}) != 0:
-        raise InputError(('Requested conformational search on a molecule that contains atoms different '
-                            'than the ones for which OpenBabel Force Fields are parametrized. Please '
-                            'perform this conformational search through the csearch> operator.'))
-                                
-    confname = filename[:-4] + '_confab.xyz'
-
-    with suppress_stdout_stderr():
-        check_call(f'obabel {filename} -O {confname} --confab --rcutoff 0.5 --original'.split(), stdout=DEVNULL, stderr=STDOUT)
-        # running Confab through Openbabel
-
-    data = read_xyz(confname)
-    conformers = data.atomcoords
-        
-    # if len(conformers) > 10 and not options.let:
-    #     conformers = conformers[0:10]
-    #     logfunction(f'Will use only the best 10 conformers for TSCoDe embed.\n')
-
-    os.remove(confname)
-    with open(confname, 'w') as f:
-        for i, conformer in enumerate(conformers):
-            write_xyz(conformer, data.atomnos, f, title=f'Generated conformer {i}')
-
-    return confname
-
 def csearch_operator(filename, embedder, keep_hb=False, mode=1):
     '''
     '''
@@ -452,17 +440,68 @@ def mtd_search_operator(filename, embedder):
     if not hasattr(mol, 'charge'):
         mol.charge = 0
 
-    assert len(mol.atomcoords) == 1, 'mtd_search> operator works with a single structure as input.'
+    if not embedder.options.let:
+        if len(mol.atomcoords) >= 20:
+            raise InputError('The mtd_search> operator was given more than 20 input structures. ' +
+                             'This would run >20 metadynamic conformational searches. If this was not a mistake, ' +
+                             'add the LET keyword an re-run the job.')
 
     logfunction = embedder.log
+    constrained_indices = _get_internal_constraints(filename, embedder)
+    constrained_distances = [embedder.get_pairing_dists_from_constrained_indices(cp) for cp in constrained_indices]
+
+    logfunction(f'--> {filename}: Geometry optimization pre-mtd_search ({embedder.options.theory_level} via {embedder.options.calculator})')
+    logfunction(f'    {len(constrained_indices)} constraints applied{": "+constrained_indices if len(constrained_indices) > 0 else ""}')
+    
+    for c, coords in enumerate(mol.atomcoords.copy()):
+        logfunction(f"    Optimizing conformer {c+1}/{len(mol.atomcoords)}")
+
+        opt_coords, _, success = optimize(
+                                    coords,
+                                    mol.atomnos,
+                                    calculator=embedder.options.calculator,
+                                    method=embedder.options.theory_level,
+                                    solvent=embedder.options.solvent,
+                                    charge=embedder.options.charge,
+                                    procs=embedder.procs,
+                                    constrained_indices=constrained_indices,
+                                    constrained_distances=constrained_distances,
+                                    title=f'{filename.split(".")[0]}_conf{c+1}',
+                                ) if embedder.options.optimization else coords
+        
+        exit_status = "" if success else "CRASHED"
+        
+        if success:
+            success = molecule_check(coords, opt_coords, mol.atomnos)
+            exit_status = "" if success else "SCRAMBLED"
+
+        if not success:
+            dumpname = filename.split(".")[0] + f"_conf{c+1}_{exit_status}.xyz"
+            with open(dumpname, "w") as f:
+                write_xyz(opt_coords, mol.atomnos, f, title=f"{filename}, conformer {c+1}/{len(mol.atomcoords)}, {exit_status}")
+
+            logfunction(f"{filename}, conformer {c+1}/{len(mol.atomcoords)} optimization {exit_status}. Inspect geometry at {dumpname}. Aborting run.")
+
+            raise FatalError(filename)
+        
+        # update embedder structures after optimization
+        mol.atomcoords[c] = opt_coords
+
+    # update mol and embedder graph after optimization 
+    mol.graph = graphize(mol.atomcoords[0], mol.atomnos)
+    embedder.graphs = [m.graph for m in embedder.objects]
 
     max_workers = embedder.avail_cpus//2 or 1
     logfunction(f'--> Performing {embedder.options.calculator} GFN2//GFN-FF' + (
-                    f'{f"/{embedder.options.solvent.upper()}" if embedder.options.solvent is not None else ""} ' +
-                    f'metadynamic conformational search on {filename} via CREST (2 cores, {max_workers} threads)'))
+                f'{f"/{embedder.options.solvent.upper()}" if embedder.options.solvent is not None else ""} ' +
+                f'metadynamic conformational search on {filename} via CREST.\n' +
+                f'    (2 cores, {max_workers} threads, {embedder.options.kcal_thresh} kcal/mol thr.)'))
+
+    if embedder.options.crestnci:
+        logfunction(f'--> CRESTNCI: Running crest in NCI mode (wall potential applied)')
     
     if len(mol.atomcoords) > 1:
-        embedder.log(f'Requested conformational search on multimolecular file - will do\n' +
+        embedder.log(f'--> Requested conformational search on multimolecular file - will do\n' +
                       'an individual search from each conformer (might be time-consuming).')
 
     t_start = time.perf_counter()
@@ -470,17 +509,38 @@ def mtd_search_operator(filename, embedder):
     for i, coords in enumerate(mol.atomcoords):
 
         t_start_conf = time.perf_counter()
-
-        conf_batch = crest_mtd_search(
-                                        coords,
-                                        mol.atomnos,
-                                        constrained_indices=_get_internal_constraints(filename, embedder),
-                                        solvent=embedder.options.solvent,
-                                        charge=mol.charge,
-                                        title=mol.rootname,
-                                        procs=2,
-                                        threads=max_workers,
-                                    )
+        try:
+            conf_batch = crest_mtd_search(
+                                            coords,
+                                            mol.atomnos,
+                                            constrained_indices=constrained_indices,
+                                            constrained_distances=constrained_distances,
+                                            solvent=embedder.options.solvent,
+                                            charge=mol.charge,
+                                            kcal=embedder.options.kcal_thresh,
+                                            ncimode=embedder.options.crestnci,
+                                            title=mol.rootname+"_mtd_csearch",
+                                            procs=2,
+                                            threads=max_workers,
+                                        )
+            
+        # if the run errors out, we retry with XTB2
+        except CalledProcessError:
+            logfunction(f'--> Metadynamics run failed with GFN2-XTB//GFN-FF, retrying with just GFN2-XTB (slower but more stable)')
+            conf_batch = crest_mtd_search(
+                                            coords,
+                                            mol.atomnos,
+                                            constrained_indices=constrained_indices,
+                                            constrained_distances=constrained_distances,
+                                            solvent=embedder.options.solvent,
+                                            charge=mol.charge,
+                                            method='GFN2-XTB', # try with XTB2
+                                            kcal=embedder.options.kcal_thresh,
+                                            ncimode=embedder.options.crestnci,
+                                            title=mol.rootname+"_mtd_csearch",
+                                            procs=2,
+                                            threads=max_workers,
+                                        )
 
         conformers.extend(conf_batch)
 
@@ -498,22 +558,26 @@ def mtd_search_operator(filename, embedder):
     quadruplets = _get_quadruplets(mol.graph)
     conformers, _ = prune_conformers_tfd(conformers, quadruplets)
 
-    ### MOI
-    conformers, _ = prune_by_moment_of_inertia(conformers, mol.atomnos)
+    # ### MOI - turned off, as it would get rid of enantiomeric conformations
+    # conformers, _ = prune_by_moment_of_inertia(conformers, mol.atomnos)
 
     ### RMSD
     if len(conformers) < 5E4:
-        conformers, _ = prune_conformers_rmsd(conformers, mol.atomnos, max_rmsd=embedder.options.rmsd)
+        conformers, _ = prune_conformers_rmsd(conformers, mol.atomnos, rmsd_thr=embedder.options.rmsd)
     if len(conformers) < 1E3:
-        conformers, _ = prune_conformers_rmsd_rot_corr(conformers, mol.atomnos, mol.graph)
+        conformers, _ = prune_conformers_rmsd_rot_corr(conformers, mol.atomnos, mol.graph, max_rmsd=embedder.options.rmsd)
 
-    embedder.log(f'  Discarded {len(conformers)-before} similar structures ({len(conformers)} left)')
+    embedder.log(f'  Discarded {before-len(conformers)} RMSD-similar structures ({len(conformers)} left)\n')
 
     ### PRINTOUT
     with open(f'{mol.rootname}_mtd_confs.xyz', 'w') as f:
         for i, new_s in enumerate(conformers):
             write_xyz(new_s, mol.atomnos, f, title=f'Conformer {i}/{len(conformers)} from CREST MTD')
 
+    
+    # check the structures again and warn if some look compenetrated
+    self.check_objects_compenetration()
+
     return f'{mol.rootname}_mtd_confs.xyz'
 
 def scan_operator(filename, embedder):
