@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ from tscode.calculators._gaussian import gaussian_opt
 from tscode.calculators._mopac import mopac_opt
 from tscode.calculators._orca import orca_opt
 from tscode.calculators._xtb import xtb_opt
-from tscode.python_functions import prune_conformers_rmsd
+from tscode.rmsd_pruning import prune_conformers_rmsd
 from tscode.settings import DEFAULT_LEVELS, FF_CALC
 from tscode.utils import (loadbar, molecule_check, pt, scramble_check,
                           time_to_string, write_xyz)
@@ -41,11 +41,6 @@ opt_funcs_dict = {
     'XTB':xtb_opt,
 }
 
-if FF_CALC in ('OB', 'OPENBABEL'):
-    from tscode.calculators._openbabel import openbabel_opt
-    opt_funcs_dict['OB'] = openbabel_opt
-    opt_funcs_dict['OPENBABEL'] = openbabel_opt
-
 def optimize(
             coords,
             atomnos,
