@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,6 +17,11 @@ GNU General Public License for more details.
 '''
 from periodictable import core, covalent_radius, mass
 
-pt = core.PeriodicTable(table="H=1")
-covalent_radius.init(pt)
-mass.init(pt)
\ No newline at end of file
+for pt_n in range(5):
+    try:
+        pt = core.PeriodicTable(table=f"H={pt_n+1}")
+        covalent_radius.init(pt)
+        mass.init(pt)
+    except ValueError:
+        continue
+    break
\ No newline at end of file
