@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -653,4 +653,9 @@ def get_atom_type(graph, index, override=None):
         return atom_type_dict[override]
 
     nb = neighbors(graph, index)
-    return atom_type_dict[pt[graph.nodes[index]['atomnos']].symbol + str(len(nb))]
\ No newline at end of file
+    code = pt[graph.nodes[index]['atomnos']].symbol + str(len(nb))
+    try:
+        return atom_type_dict[code]
+
+    except KeyError:
+        raise KeyError(f'Orbital type {code} not known (index {index})')
\ No newline at end of file
