@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -90,7 +90,7 @@ def run_tests():
                 print('Module imported successfully.')
 
             except ImportError:
-                raise Exception(f'Could not import OpenBabel Python module. Is standalone openbabel correctly installed?')
+                raise Exception(f'Could not import OpenBabel Python module. Is standalone/conda openbabel correctly installed?')
         else: # == 'XTB', 'GAUSSIAN'
             opt_funcs_dict[FF_CALC](data.atomcoords[0],
                                     data.atomnos,
