@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -1153,6 +1153,9 @@ def prune_conformers_rmsd_rot_corr(structures, atomnos, graph, max_rmsd=0.25, ve
 
     # restore molecular graph
     for hb in hydrogen_bonds:
-        graph.remove_edge(*hb)
+        try:
+            graph.remove_edge(*hb)
+        except nx.NetworkXError():
+            pass
 
     return structures[final_mask], final_mask
\ No newline at end of file
