@@ -2,7 +2,7 @@
 '''
 
 TSCODE: Transition State Conformational Docker
-Copyright (C) 2021-2023 Nicolò Tampellini
+Copyright (C) 2021-2024 Nicolò Tampellini
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ GNU General Public License for more details.
 import os
 import sys
 import time
+from shutil import rmtree
 from subprocess import DEVNULL, STDOUT, CalledProcessError, check_call, run
 
 import numpy as np
@@ -75,6 +76,8 @@ def clean_directory(to_remove=None):
         for name in to_remove:
             try:
                 os.remove(name)
+            except IsADirectoryError:
+                rmtree(os.path.join(os.getcwd(), name))
             except FileNotFoundError:
                 pass
 
@@ -82,11 +85,15 @@ def clean_directory(to_remove=None):
         if f.split('.')[0] == 'temp':
             try:
                 os.remove(f)
+            except IsADirectoryError:
+                rmtree(os.path.join(os.getcwd(), f))
             except FileNotFoundError:
                 pass
         elif f.startswith('temp_'):
             try:
                 os.remove(f)
+            except IsADirectoryError:
+                rmtree(os.path.join(os.getcwd(), f))
             except FileNotFoundError:
                 pass
 
@@ -345,7 +352,7 @@ def molecule_check(old_coords, new_coords, atomnos, max_newbonds=0):
 
     return True
 
-def scramble_check(TS_structure, TS_atomnos, excluded_atoms, mols_graphs, max_newbonds=0) -> bool:
+def scramble_check(TS_structure, TS_atomnos, excluded_atoms, mols_graphs, max_newbonds=0, logfunction=False, title=None) -> bool:
     '''
     Check if a multimolecular arrangement has scrambled during some optimization
     steps. If more than a given number of bonds changed (formed or broke) the
@@ -373,6 +380,8 @@ def scramble_check(TS_structure, TS_atomnos, excluded_atoms, mols_graphs, max_ne
     # removing bonds involving constrained atoms: they are not counted as scrambled bonds
 
     if len(delta_bonds) > max_newbonds:
+        if logfunction is not None:
+            logfunction(f"{title}, scramble_check - found {len(delta_bonds)} extra bonds: {delta_bonds}")
         return False
 
     return True
@@ -440,13 +449,51 @@ def smi_to_3d(smi, new_filename):
 
     return new_filename + ".xyz"
 
-def timing_wrapper(function, *args, **kwargs):
+def timing_wrapper(function, *args, payload=None, **kwargs):
     '''
     Generic function wrapper that appends the
     execution time at the end of return.
+    If payload is not None, appends it at the end
+    of the function return, before the elapsed time.
     
     '''
     start_time = time.perf_counter()
     func_return = function(*args, **kwargs)
     elapsed = time.perf_counter() - start_time
-    return func_return, elapsed
\ No newline at end of file
+
+    if payload is None:
+        return func_return, elapsed
+    
+    return func_return, payload, elapsed
+
+def _saturation_check(atomnos, charge=0):
+
+    transition_metals = [
+                    "Sc", "Ti", "V", "Cr", "Mn", "Fe",
+                    "Co", "Ni", "Cu", "Zn", "Y", "Zr",
+                    "Nb", "Mo", "Tc", "Ru", "Rh", "Pd",
+                    "Ag", "Cd", "La", "Ce", "Pr", "Nd",
+                    "Pm", "Sm", "Eu", "Gd", "Tb", "Dy",
+                    "Ho", "Er", "Tm", "Yb", "Lu", "Hf",
+                    "Ta", "W", "Re", "Os", "Ir", "Pt",
+                    "Au", "Hg", "Th", "Pa", "U", "Np",
+                    "Pu", "Am", 
+    ]
+
+    # if we have any transition metal, it's hard to tell
+    # if the structure looks ok: in this case we assume it is.
+    organometallic = any([pt[a].symbol in transition_metals for a in atomnos])
+
+    odd_valent = [  #1 valent
+                    "H", "Li", "Na", "K", "Rb", "Cs",
+                    "F", "Cl", "Br", "I", "At",
+
+                    # 3/5 valent
+                    "N", "P", "As", "Sb", "Bi",
+                    "B", "Al", "Ga", "In", "Tl",
+                 ]
+
+    n_odd_valent = sum([1 for a in atomnos if pt[a].symbol in odd_valent])
+    looks_ok = ((n_odd_valent + charge) / 2) % 1 < 0.001 if not organometallic else True
+
+    return looks_ok
